# -*- coding: utf-8 -*-


def test_config(testdir):
    """Make sure that pytest accepts the `cookies` fixture."""

    # create a temporary pytest test module
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        import yaml


        def test_user_dir(tmpdir_factory, _cookiecutter_config_file):
            basetemp = tmpdir_factory.getbasetemp()

            assert _cookiecutter_config_file.basename == 'config'

            user_dir = _cookiecutter_config_file.dirpath()
            assert user_dir.fnmatch('user_dir?')

            assert user_dir.dirpath() == basetemp


        def test_valid_cookiecutter_config(_cookiecutter_config_file):
            with open(_cookiecutter_config_file) as f:
                config = yaml.load(f, Loader=yaml.Loader)

            user_dir = _cookiecutter_config_file.dirpath()

            expected = {
                'cookiecutters_dir': str(user_dir.join('cookiecutters')),
                'replay_dir': str(user_dir.join('cookiecutter_replay')),
            }
            assert config == expected
    """
    )

    # run pytest with the following cmd args
    result = testdir.runpytest("-v")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        ["*::test_user_dir PASSED*", "*::test_valid_cookiecutter_config PASSED*"]
    )

    # make sure that that we get a '0' exit code for the testsuite
    assert result.ret == 0



# -*- coding: utf-8 -*-

import collections
import json

import pytest

pytest_plugins = "pytester"


@pytest.fixture(name="cookiecutter_template")
def fixture_cookiecutter_template(tmpdir):
    template = tmpdir.ensure("cookiecutter-template", dir=True)

    template_config = collections.OrderedDict(
        [("repo_name", "foobar"), ("short_description", "Test Project")]
    )
    template.join("cookiecutter.json").write(json.dumps(template_config))

    template_readme = "\n".join(
        [
            "{{cookiecutter.repo_name}}",
            "{% for _ in cookiecutter.repo_name %}={% endfor %}",
            "{{cookiecutter.short_description}}",
        ]
    )

    repo = template.ensure("{{cookiecutter.repo_name}}", dir=True)
    repo.join("README.rst").write(template_readme)

    return template



# -*- coding: utf-8 -*-

import json
import pytest


def test_cookies_fixture(testdir):
    """Make sure that pytest accepts the `cookies` fixture."""

    # create a temporary pytest test module
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_valid_fixture(cookies):
            assert hasattr(cookies, 'bake')
            assert callable(cookies.bake)
    """
    )

    # run pytest with the following cmd args
    result = testdir.runpytest("-v")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(["*::test_valid_fixture PASSED*"])

    # make sure that that we get a '0' exit code for the testsuite
    assert result.ret == 0


def test_cookies_bake_with_template_kwarg(testdir, cookiecutter_template):
    """bake accepts a template kwarg."""
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_bake_project(cookies):
            result = cookies.bake(
                extra_context={'repo_name': 'helloworld'},
                template=r'%s',
            )

            assert result.exit_code == 0
            assert result.exception is None
            assert result.project.basename == 'helloworld'
            assert result.project.isdir()

            assert str(result) == '<Result {}>'.format(result.project)
    """
        % cookiecutter_template
    )

    # run pytest without the template cli arg
    result = testdir.runpytest("-v")

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])


def test_cookies_bake_template_kwarg_overrides_cli_option(
    testdir, cookiecutter_template
):
    """bake template kwarg overrides cli option."""

    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_bake_project(cookies):
            result = cookies.bake(
                extra_context={'repo_name': 'helloworld'},
                template=r'%s',
            )

            assert result.exit_code == 0
            assert result.exception is None
            assert result.project.basename == 'helloworld'
            assert result.project.isdir()

            assert str(result) == '<Result {}>'.format(result.project)
    """
        % cookiecutter_template
    )

    # run pytest with a bogus template name
    # it should use template directory passed to `cookies.bake`
    result = testdir.runpytest("-v", "--template=foobar")

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])


def test_cookies_bake(testdir, cookiecutter_template):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it.
    """
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_bake_project(cookies):
            result = cookies.bake(extra_context={'repo_name': 'helloworld'})

            assert result.exit_code == 0
            assert result.exception is None

            assert result.project_path.name == 'helloworld'
            assert result.project_path.is_dir()
            assert str(result) == '<Result {}>'.format(result.project_path)

            assert result.project.basename == 'helloworld'
            assert result.project.isdir()
            assert str(result) == '<Result {}>'.format(result.project)
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(cookiecutter_template))

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])


def test_cookies_bake_project_warning(testdir, cookiecutter_template):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it and check for warnings when accesssing the project
    attribute.
    """
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-
        import warnings

        def test_bake_project(cookies):
            warning_message = (
                "project is deprecated and will be removed in a future release, "
                "please use project_path instead."
            )

            result = cookies.bake(extra_context={'repo_name': 'helloworld'})

            assert result.exit_code == 0
            assert result.exception is None

            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")
                project_basename = result.project.basename

            assert project_basename == 'helloworld'

            [warning] = w
            assert issubclass(warning.category, DeprecationWarning)
            assert str(warning.message) == warning_message

            assert str(result) == '<Result {}>'.format(result.project)
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(cookiecutter_template))

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])


def test_cookies_bake_result_context(testdir, cookiecutter_template):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it.

    Check that the result holds the rendered context.
    """

    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        import collections

        def test_bake_project(cookies):
            result = cookies.bake(extra_context=collections.OrderedDict([
                ('repo_name', 'cookies'),
                ('short_description', '{{cookiecutter.repo_name}} is awesome'),
            ]))

            assert result.exit_code == 0
            assert result.exception is None
            assert result.project.basename == 'cookies'
            assert result.project.isdir()

            assert result.context == {
                'repo_name': 'cookies',
                'short_description': 'cookies is awesome',
            }

            assert str(result) == '<Result {}>'.format(result.project)
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(cookiecutter_template))

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])


def test_cookies_bake_result_context_exception(testdir, cookiecutter_template):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it.

    Check that exceptions resulting from rendering the context are stored on
    result and that the rendered context is not set.
    """

    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        import collections

        def test_bake_project(cookies):
            result = cookies.bake(extra_context=collections.OrderedDict([
                ('repo_name', 'cookies'),
                ('short_description', '{{cookiecutter.nope}}'),
            ]))

            assert result.exit_code == -1
            assert result.exception is not None
            assert result.project is None

            assert result.context is None

            assert str(result) == '<Result {!r}>'.format(result.exception)
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(cookiecutter_template))

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])


def test_cookies_bake_should_create_new_output_directories(
    testdir, cookiecutter_template
):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it.
    """
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_bake_should_create_new_output(cookies):
            first_result = cookies.bake()
            assert first_result.exception is None
            assert first_result.project.dirname.endswith('bake00')

            second_result = cookies.bake()
            assert second_result.exception is None
            assert second_result.project.dirname.endswith('bake01')
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(cookiecutter_template))

    result.stdout.fnmatch_lines(["*::test_bake_should_create_new_output PASSED*"])


def test_cookies_fixture_removes_output_directories(testdir, cookiecutter_template):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it.
    """
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-
        import os

        def test_to_create_result(cookies):
            global result_dirname
            result = cookies.bake()
            result_dirname = result.project.dirname
            assert result.exception is None

        def test_previously_generated_directory_is_removed(cookies):
            exists = os.path.isdir(result_dirname)
            assert exists is False
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(cookiecutter_template))

    result.stdout.fnmatch_lines(
        [
            "*::test_to_create_result PASSED*",
            "*::test_previously_generated_directory_is_removed PASSED*",
        ]
    )


def test_cookies_fixture_doesnt_remove_output_directories(
    testdir, cookiecutter_template
):
    """Programmatically create a **Cookiecutter** template and use `bake` to
    create a project from it.
    """
    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-
        import os

        def test_to_create_result(cookies):
            global result_dirname
            result = cookies.bake()
            result_dirname = result.project.dirname
            assert result.exception is None

        def test_previously_generated_directory_is_not_removed(cookies):
            exists = os.path.isdir(result_dirname)
            assert exists is True
    """
    )

    result = testdir.runpytest(
        "-v", "--template={}".format(cookiecutter_template), "--keep-baked-projects"
    )

    result.stdout.fnmatch_lines(
        [
            "*::test_to_create_result PASSED*",
            "*::test_previously_generated_directory_is_not_removed PASSED*",
        ]
    )


def test_cookies_bake_should_handle_exception(testdir):
    """Programmatically create a **Cookiecutter** template and make sure that
    cookies.bake() handles exceptions that happen during project generation.

    We expect **Cookiecutter** to raise a `NonTemplatedInputDirException`.
    """
    template = testdir.tmpdir.ensure("cookiecutter-fail", dir=True)

    template_config = {"repo_name": "foobar", "short_description": "Test Project"}
    template.join("cookiecutter.json").write(json.dumps(template_config))

    template.ensure("cookiecutter.repo_name", dir=True)

    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_bake_should_fail(cookies):
            result = cookies.bake()

            assert result.exit_code == -1
            assert result.exception is not None
            assert result.project is None
    """
    )

    result = testdir.runpytest("-v", "--template={}".format(template))

    result.stdout.fnmatch_lines(["*::test_bake_should_fail PASSED*"])


@pytest.mark.parametrize("choice", ["mkdocs", "sphinx", "none"])
def test_cookies_bake_choices(testdir, choice):
    """Programmatically create a **Cookiecutter** template and make sure that
    cookies.bake() works with choice variables.
    """
    template = testdir.tmpdir.ensure("cookiecutter-choices", dir=True)
    template_config = {"repo_name": "docs", "docs_tool": ["mkdocs", "sphinx", "none"]}
    template.join("cookiecutter.json").write(json.dumps(template_config))

    repo = template.ensure("{{cookiecutter.repo_name}}", dir=True)
    repo.join("README.rst").write("docs_tool: {{cookiecutter.docs_tool}}")

    testdir.makepyfile(
        """
        # -*- coding: utf-8 -*-

        def test_bake_project(cookies):
            result = cookies.bake(
                extra_context={'docs_tool': '%s'},
                template=r'%s',
            )

            assert result.exit_code == 0
            assert result.exception is None
            assert result.project.basename == 'docs'
            assert result.project.isdir()

            assert result.project.join('README.rst').read() == 'docs_tool: %s'

            assert str(result) == '<Result {}>'.format(result.project)
    """
        % (choice, template, choice)
    )

    # run pytest without the template cli arg
    result = testdir.runpytest("-v")

    result.stdout.fnmatch_lines(["*::test_bake_project PASSED*"])

    result = testdir.runpytest("-v", "--template={}".format(template))



# -*- coding: utf-8 -*-


def test_cookies_group(testdir):
    result = testdir.runpytest("--help")
    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(["cookies:", "*--template=TEMPLATE*"])



# -*- coding: utf-8 -*-

import os
import pathlib
import warnings

import py
import pytest
import yaml

from cookiecutter.generate import generate_context
from cookiecutter.main import cookiecutter
from cookiecutter.prompt import prompt_for_config


class Result(object):
    """Holds the captured result of the cookiecutter project generation."""

    def __init__(self, exception=None, exit_code=0, project_dir=None, context=None):
        self.exception = exception
        self.exit_code = exit_code
        self.context = context
        self._project_dir = project_dir

    @property
    def project(self):
        """Return a py.path.local object if no exception occurred."""
        warning_message = (
            "project is deprecated and will be removed in a future release, "
            "please use project_path instead."
        )

        warnings.warn(
            warning_message,
            DeprecationWarning,
            stacklevel=1,
        )

        if self.exception is None:
            return py.path.local(self._project_dir)

        return None

    @property
    def project_path(self):
        """Return a pathlib.Path object if no exception occurred."""

        if self.exception is None:
            return pathlib.Path(self._project_dir)

        return None

    def __repr__(self):
        if self.exception:
            return "<Result {!r}>".format(self.exception)

        return "<Result {}>".format(self.project)


class Cookies(object):
    """Class to provide convenient access to the cookiecutter API."""

    def __init__(self, template, output_factory, config_file):
        self._default_template = template
        self._output_factory = output_factory
        self._config_file = config_file
        self._counter = 0

    def _new_output_dir(self):
        dirname = "bake{:02d}".format(self._counter)
        output_dir = self._output_factory(dirname)
        self._counter += 1
        return output_dir

    def bake(self, extra_context=None, template=None):
        exception = None
        exit_code = 0
        project_dir = None
        context = None

        if template is None:
            template = self._default_template

        context_file = pathlib.Path(template) / "cookiecutter.json"

        try:
            # Render the context, so that we can store it on the Result
            context = prompt_for_config(
                generate_context(
                    context_file=str(context_file), extra_context=extra_context
                ),
                no_input=True,
            )

            # Run cookiecutter to generate a new project
            project_dir = cookiecutter(
                template,
                no_input=True,
                extra_context=extra_context,
                output_dir=str(self._new_output_dir()),
                config_file=str(self._config_file),
            )
        except SystemExit as e:
            if e.code != 0:
                exception = e
            exit_code = e.code
        except Exception as e:
            exception = e
            exit_code = -1

        return Result(
            exception=exception,
            exit_code=exit_code,
            project_dir=project_dir,
            context=context,
        )


@pytest.fixture(scope="session")
def _cookiecutter_config_file(tmpdir_factory):
    user_dir = tmpdir_factory.mktemp("user_dir")
    config_file = user_dir.join("config")

    config = {
        "cookiecutters_dir": str(user_dir.mkdir("cookiecutters")),
        "replay_dir": str(user_dir.mkdir("cookiecutter_replay")),
    }

    with config_file.open("w", encoding="utf-8") as f:
        yaml.dump(config, f, Dumper=yaml.Dumper)

    return config_file


@pytest.fixture
def cookies(request, tmpdir, _cookiecutter_config_file):
    """Yield an instance of the Cookies helper class that can be used to
    generate a project from a template.

    Run cookiecutter:
        result = cookies.bake(extra_context={
            'variable1': 'value1',
            'variable2': 'value2',
        })
    """
    template_dir = request.config.option.template

    output_dir = tmpdir.mkdir("cookies")
    output_factory = output_dir.mkdir

    yield Cookies(template_dir, output_factory, _cookiecutter_config_file)

    # Add option to keep generated output directories.
    if not request.config.option.keep_baked_projects:
        output_dir.remove()


@pytest.fixture(scope="session")
def cookies_session(request, tmpdir_factory, _cookiecutter_config_file):
    """Yield an instance of the Cookies helper class that can be used to
    generate a project from a template.

    Run cookiecutter:
        result = cookies.bake(extra_context={
            'variable1': 'value1',
            'variable2': 'value2',
        })
    """
    template_dir = request.config.option.template

    output_dir = tmpdir_factory.mktemp("cookies")
    output_factory = output_dir.mkdir

    yield Cookies(template_dir, output_factory, _cookiecutter_config_file)

    # Add option to keep generated output directories.
    if not request.config.option.keep_baked_projects:
        output_dir.remove()


def pytest_addoption(parser):
    group = parser.getgroup("cookies")
    group.addoption(
        "--template",
        action="store",
        default=".",
        dest="template",
        help="specify the template to be rendered",
        type=str,
    )

    group.addoption(
        "--keep-baked-projects",
        action="store_true",
        default=False,
        dest="keep_baked_projects",
        help="Keep projects directories generated with 'cookies.bake()'.",
    )


def pytest_configure(config):
    # To protect ourselves from tests or fixtures changing directories, keep
    # an absolute path to the template.
    config.option.template = os.path.abspath(config.option.template)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import os
import setuptools


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


setuptools.setup(
    name="pytest-cookies",
    version="0.7.0",
    author="Raphael Pierzina",
    author_email="raphael@hackebrot.de",
    maintainer="Raphael Pierzina",
    maintainer_email="raphael@hackebrot.de",
    license="MIT",
    url="https://github.com/hackebrot/pytest-cookies",
    project_urls={
        "Repository": "https://github.com/hackebrot/pytest-cookies",
        "Issues": "https://github.com/hackebrot/pytest-cookies/issues",
    },
    description="The pytest plugin for your Cookiecutter templates. 🍪",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
    install_requires=[
        "cookiecutter>=2.1.0",  # uses pyyaml
        "pytest>=3.9.0",  # adds tmp_path fixtures
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python",
        "Topic :: Software Development :: Testing",
        "Framework :: Pytest",
    ],
    entry_points={"pytest11": ["cookies = pytest_cookies.plugin"]},
    keywords=["cookiecutter", "pytest"],
)
